@@ -2,7 +2,6 @@ context("Compare ordinalNet results against other packages.")
 MASSInst <- require(MASS)
 glmnetInst <- require(glmnet)
 penalizedInst <- require(penalized)
-glmnetcrInst <- require(glmnetcr) && packageVersion("glmnetcr") >= "1.0.3"
 VGAMInst <- require(VGAM)
 rmsInst <- require(rms)
 
@@ -180,43 +179,9 @@ test_that("Binary logistic regression with positive constraints matches penalize
     expect_equal(coef(o), c(-pp@unpenalized, pp@penalized), check.attributes=FALSE)
 })
 
-test_that("Elastic net sratio matches glmnetcr::glmnet.cr", {
-    if (!glmnetcrInst) skip("glmnetcr >= 1.0.3 not installed")
-    # Unpenalized sratio (do not run: glmnetcr cannot fit unpenalized model as of version 1.0.5)
-    # o <- ordinalNet(x, y, lambdaVals=0, family="sratio", link="logit")
-    # g <- glmnetcr(x, y, lambda=0, method="forward")
-    # coefg <- with(coef(g, s=1), (c(tail(beta, k) + a0, head(beta, -k))))
-    # expect_equal(coef(o), coefg, check.attributes=FALSE, tolerance=1e-2)
-    # rm(o, g, coefg)
-    # Unpenalized sratio, reverse (do not run: glmnetcr cannot fit unpenalized model as of version 1.0.5)
-    # o <- ordinalNet(x, y, lambdaVals=0, family="sratio", link="logit", reverse=TRUE)
-    # g <- glmnetcr(x, y, lambda=0, method="backward")
-    # coefg <- with(coef(g, s=1), (c(rev(tail(beta, k)) + a0, head(beta, -k))))
-    # expect_equal(coef(o), coefg, check.attributes=FALSE, tolerance=.02)
-    # rm(o, g, coefg)
-    # Penalized sratio
-    # Matches reasonably well, but difference is more than just convergence error
-    # Not sure why glmnet.cr lambda needs to be multiplied by about 0.5 to match.
-    # Standardizing covariates does not change this, and it's the same for alpha=0 and alpha=1
-    o <- ordinalNet(x, y, alpha=.5, lambdaVals=.1, family="sratio", link="logit")
-    g <- glmnetcr(x, y, alpha=.5, lambda=.1/2, method="forward", exclude=NULL)
-    coefg <- with(coef(g, s=1), (c(tail(beta, k) + a0, head(beta, -k))))
-    expect_equal(coef(o), coefg, check.attributes=FALSE, tolerance=.1)
-    rm(o, g, coefg)
-    # Penalized sratio, reverse
-    # Matches reasonably well, but difference is more than just convergence error
-    # Not sure why glmnet.cr lambda needs to be multiplied by about 0.5 to match.
-    # Standardizing covariates does not change this, and it's the same for alpha=0 and alpha=1
-    o <- ordinalNet(x, y, alpha=.5, lambdaVals=.1, family="sratio", link="logit", reverse=TRUE)
-    g <- glmnetcr(x, y, alpha=.5, lambda=.1/2, method="backward", exclude=NULL)
-    coefg <- with(coef(g, s=1), (c(rev(tail(beta, k)) + a0, head(beta, -k))))
-    expect_equal(coef(o), coefg, check.attributes=FALSE, tolerance=.1)
-    rm(o, g, coefg)
-})
-
 test_that("Cumulative logit ridge matches rms::lrm", {
     if (!rmsInst) skip("rms not installed.")
     m1 <- ordinalNet(x, y, alpha=0, lambdaVals=.1)
-    m2 <- lrm(y~x, penalty=.1*n)
+    m2 <- lrm(y~., data=data.frame(y, x), penalty=.1*n)
     expect_equal(coef(m1), -coef(m2), check.attributes=FALSE, tolerance=1e-2)
 })
